import logging

import re


def validate_id(id):
    if isinstance(id, str):
        r = re.compile("(^(?!\-)[a-z0-9\-]+(?<!\-)$)")
        return id if r.match(id) else False


logger = logging.getLogger(__name__)

data_types = [
    "integer",
    "float",
    "boolean",
    "string",
    "enum",
    "color",
]

"""
meta dict

meta = {
    'mainkeyid1' : {
        'name'  : name,
        'value' : value,
        'subkeys' = {
            'subkeyid1' : {
                'key' : name},
                'value' : value,
            },
        },
    },
    'mainkeyid2' : {
        'name'  : name,
        'value' : value,
    },
}
"""


class Property_Base(object):
    def __init__(
        self,
        node,
        id,
        name=None,
        settable=False,
        retained=True,
        qos=1,
        unit=None,
        data_type=None,
        data_format=None,
        value=None,
        set_value=None,
        tags=[],
        meta={},
    ):
        if validate_id(id) is False:
            logger.error("Property ID not valid {}".format(id))
            assert validate_id(id), "Property ID is not valid {}".format(id)

        self.id = id
        self.name = name
        self.retained = retained
        self.qos = qos
        self.unit = unit
        self.node = node
        self.topic = node.topic
        self.tags = tags
        self.meta = meta

        assert data_type in data_types
        self.data_type = data_type
        self.data_format = data_format

        self.settable = settable
        if settable:  # must provide a function to call to set the value
            assert set_value
            self.set_value = set_value

        self._value = value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value, retain=True, qos=1):
        if self.validate_value(value):
            self._value = value
            self.publish(self.topic, self.get_payload_from_value(value), retain, qos)
            logger.debug("Value set to:   {}".format(value))
        else:
            logger.warn(
                "Invalid Value: Device {} Node {} Property {} Value {}".format(
                    self.node.device.name, self.node.name, self.name, value
                )
            )

    def validate_value(self, value):
        return True  # override as needed

    def get_value_from_payload(self, payload):  # convert payload to a value we can use
        return payload

    def get_payload_from_value(self, value):  # convert value to a payload for homie
        return value

    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, parent_topic):
        self._topic = "/".join([parent_topic, self.id])

    def publish(self, topic, payload, retain, qos):
        self.node.property_publisher(topic, payload, retain, qos)

    def publish_attributes(self, retain=True, qos=1):
        self.publish("/".join((self.topic, "$name")), self.name, retain, qos)
        self.publish(
            "/".join((self.topic, "$settable")), str(self.settable).lower(), retain, qos
        )
        self.publish(
            "/".join((self.topic, "$retained")), str(self.retained).lower(), retain, qos
        )
        if self.unit:
            self.publish("/".join((self.topic, "$unit")), self.unit, retain, qos)
        if self.data_type:
            self.publish(
                "/".join((self.topic, "$datatype")), self.data_type, retain, qos
            )
        if self.data_format:
            self.publish(
                "/".join((self.topic, "$format")), self.data_format, retain, qos
            )

        if self.value is not None:  # publish value if known, by setting it
            self.value = self.value

        self.publish_tags(retain, qos)
        self.publish_meta(retain, qos)

    def publish_tags(self, retain=True, qos=1):
        tags = ",".join(self.tags)
        if tags is not "":
            self.publish("/".join((self.topic, "$tags")), tags, retain, qos)

    def publish_meta(self, retain=True, qos=1):
        main_keys = []
        for key in self.meta:
            main_keys.append(key)

        if len(main_keys) > 0:
            self.publish(
                "/".join((self.topic, "$meta", "$mainkey-ids")),
                ",".join(main_keys),
                retain,
                qos,
            )

        for main_key in main_keys:
            settings = self.meta[main_key]
            self.publish(
                "/".join((self.topic, "$meta", main_key, "$key")),
                settings["name"],
                retain,
                qos,
            )
            self.publish(
                "/".join((self.topic, "$meta", main_key, "$value")),
                settings["value"],
                retain,
                qos,
            )

            if "subkeys" in settings:
                sub_keys = []
                for key in settings["subkeys"]:
                    sub_keys.append(key)

                self.publish(
                    "/".join((self.topic, "$meta", main_key, "$subkey-ids")),
                    ",".join(sub_keys),
                    retain,
                    qos,
                )

                for sub_key in sub_keys:
                    self.publish(
                        "/".join((self.topic, "$meta", main_key, sub_key, "$key")),
                        settings["subkeys"][sub_key]["name"],
                        retain,
                        qos,
                    )
                    self.publish(
                        "/".join((self.topic, "$meta", main_key, sub_key, "$value")),
                        settings["subkeys"][sub_key]["value"],
                        retain,
                        qos,
                    )

    def get_subscriptions(self):  # subscribe to the set topic
        if self.settable:
            return {"/".join((self.topic, "set")): self.set_message_handler}
        else:
            return {}

    def set_message_handler(self, topic, payload):
        logger.info(
            "MQTT Property Message:  Topic {}, Payload {}".format(topic, payload)
        )
        self.process_set_message(topic, payload)

    def process_set_message(self, topic, payload):  # override as needed
        value = self.get_value_from_payload(payload)

        if value is not None:
            if self.validate_value(value):
                self.value = value
                self.set_value(value)  # call function to actually change the value
            else:
                logger.warning(
                    "Payload value not valid for property for topic {}, payload is {}".format(
                        topic, payload
                    )
                )
        else:
            logger.warning(
                "Unable to convert payload for property topic {}, payload is {}".format(
                    topic, payload
                )
            )




from .property_integer import Property_Integer

tags = ["Lighting", "Dimmable"]


class Property_Dimmer(Property_Integer):
    def __init__(
        self,
        node,
        id="dimmer",
        name="Dimmer",
        settable=True,
        retained=True,
        qos=1,
        unit="%",
        data_type=None,
        data_format="0:100",
        value=None,
        set_value=None,
        tags=tags,
        meta={},
    ):

        super().__init__(
            node,
            id,
            name,
            settable,
            retained,
            qos,
            unit,
            data_type,
            data_format,
            value,
            set_value,
            tags,
            meta,
        )




#!/usr/bin/env python


"""

Base MQTT Client for a Homie device

To allow for easy use of different MQTT clients

"""
from homie.support.network_information import Network_Information

network_info = Network_Information()

import logging

logger = logging.getLogger(__name__)



class MQTT_Base(object):
    def __init__(self, mqtt_settings, last_will):
        logger.debug("MQTT client Settings {}".format(mqtt_settings))

        self.last_will = last_will

        self.using_shared_mqtt_client = mqtt_settings["MQTT_SHARE_CLIENT"]

        self.mqtt_settings = mqtt_settings

        self._mqtt_connected = False

        self.ip_address = None
        self.mac_address = None

        self.homie_devices = []

    @property
    def mqtt_connected(self):
        return self._mqtt_connected

    @mqtt_connected.setter
    def mqtt_connected(self, connected):
        if connected != self._mqtt_connected:
            logger.debug("MQTT Connected is {} ".format(connected))
            self._mqtt_connected = connected
            for device in self.homie_devices:
                if device.start_time is not None:
                    device.mqtt_on_connection(connected)

    def connect(
        self,
    ):  # called by the device when its ready for the mqtt client to start, subclass to provide
        logger.debug(
            "MQTT Connecting to {} as client {}".format(
                self.mqtt_settings["MQTT_BROKER"], self.mqtt_settings["MQTT_CLIENT_ID"]
            )
        )

    def publish(self, topic, payload, retain, qos):  # subclass to provide
        logger.debug(
            "MQTT publish topic: {}, payload: {}, retain {}, qos {}".format(
                topic, payload, retain, qos
            )
        )

    def subscribe(self, topic, qos):  # subclass to provide
        logger.debug("MQTT subscribe  topic: {}, qos {}".format(topic, qos))

    def unsubscribe(self, topic):  # subclass to provide
        logger.debug("MQTT unsubscribe  topic: {}".format(topic))

    def set_will(self, will, topic, retain, qos):  # subclass to provide
        logger.info("MQTT set will {}, topic {}".format(will, topic))

    def get_mac_ip_address(self):
        if self.ip_address is None:
            self.ip_address = network_info.get_local_ip(
                self.mqtt_settings["MQTT_BROKER"], self.mqtt_settings["MQTT_PORT"]
            )

        if self.mac_address is None:
            self.mac_address = network_info.get_local_mac_for_ip(self.ip_address)

        return self.mac_address, self.ip_address

    def _on_message(self, topic, payload, retain, qos):
        logger.debug(
            "MQTT On Message: Topic {}, Payload {} Reatin {} QOS {}".format(
                topic, payload, retain, qos
            )
        )
        for device in self.homie_devices:
            if device.start_time is not None:  # device is ready
                try:
                    device.mqtt_on_message(topic, payload, retain == 1, qos)
                except:
                    logger.exception("on_message error")

    def _on_disconnect(self, rc):
        logger.warning("MQTT On Disconnect: Result Code {}".format(rc))
        self.mqtt_connected = False

    def add_device(self, device):
        self.homie_devices.append(device)

    def remove_device(self, device):  # not tested
        del self.homie_devices[device]

    def close(self):
        logger.info("MQTT Closing")



#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_base import Node_Base
from homie.node.property.property_button import Property_Button

import logging

logger = logging.getLogger(__name__)
logger.setLevel("INFO")

class Device_Button(Device_Base):
    def __init__(
        self,
        device_id=None,
        name=None,
        homie_settings=None,
        mqtt_settings=None,
    ):

        super().__init__(device_id, name, homie_settings, mqtt_settings)

        node = Node_Base(self, "button", "Button", "button")
        self.add_node(node)

        self.button = Property_Button(
                node,
                id="button",
                name="Button",
            )
        node.add_property(self.button)

        self.start()

    def push_button(self):
        self.button.push()



from homie.support.helpers import validate_id


class Node_Base(object):
    def __init__(self, device, id, name, type_, retain=True, qos=1):
        assert validate_id(id), "Node ID {} is not valid".format(id)
        assert device

        self.id = id
        self.name = name
        self.type = type_
        self.device = device

        self.retain = retain
        self.qos = qos

        self.properties = {}

        self.topic = self.device.topic

        self.published = False

    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, parent_topic):
        self._topic = "/".join([parent_topic, self.id])

    def add_property(self, property_):
        # assert self.properties [property_.id] == None
        assert property_.id not in self.properties

        self.properties[property_.id] = property_

        if self.device.state=="ready":
            self.device.state="init"
            self.publish_properties()
            self.device.state="ready"

    def remove_property(self, property_id):
        property_ = self.properties[property_id]
        del self.properties[property_id]

        if self.device.state=="ready":
#        if self.device.start_time is not None:  # running, publish property changes
            self.device.state="init"
            self.publish_properties()
            property_.publish_attributes(False, 1)
            self.device.state="ready"

    def get_property(self, property_id):
        if property_id in self.properties:
            return self.properties[property_id]
        else:
            return None

    def set_property_value(self, property_id, value):
        self.get_property(property_id).value = value

    def publish(self, topic, payload, retain, qos):
        self.device.publish(topic, payload, retain, qos)

    def property_publisher(
        self, topic, payload, retain, qos
    ):  # properties use this to publish
        if self.published:  # only publish if the node has been published
            self.device.publish(topic, payload, retain, qos)

    def publish_attributes(self, retain=True, qos=1):
        self.publish("/".join((self.topic, "$name")), self.name, retain, qos)
        self.publish("/".join((self.topic, "$type")), self.type, retain, qos)

        self.publish_properties()

    def publish_properties(self, retain=True, qos=1):
        properties = ",".join(self.properties.keys())
        self.publish("/".join((self.topic, "$properties")), properties, retain, qos)

        self.published = True  # node basics published

        for _, property_ in self.properties.items():
            # print ('NODE PUBLISH PROP ',property_.name)
            property_.publish_attributes(retain, qos)

    def get_subscriptions(self):
        subscriptions = {}

        for _, property_ in self.properties.items():
            subscriptions.update(property_.get_subscriptions())

        return subscriptions




#!/usr/bin/env python

# not fully working

import asyncio
import os
import signal
import time
import threading
import functools


from gmqtt import Client as MQTTClient
from gmqtt.mqtt.constants import MQTTv311

from homie.mqtt.mqtt_base import MQTT_Base

import logging

logger = logging.getLogger(__name__)



class GMQTT_Client(MQTT_Base):
    def __init__(self, mqtt_settings):
        MQTT_Base.__init__(self, mqtt_settings)

        self.mqtt_client = None

    def connect(self):
        MQTT_Base.connect(self)

        self.mqtt_client = MQTTClient(
            'gmqtt'#self.mqtt_settings["MQTT_CLIENT_ID"]
        )

        self.mqtt_client.on_connect = self._on_connect
        self.mqtt_client.on_message = self._on_message
        self.mqtt_client.on_disconnect = self._on_disconnect


        if self.mqtt_settings["MQTT_USERNAME"]:
            self.mqtt_client.set_auth_credentials(
                self.mqtt_settings["MQTT_USERNAME"],
                self.mqtt_settings["MQTT_PASSWORD"],
            )


        def start():
            try:
                logger.warning ('Connecting to MQTT')
                asyncio.set_event_loop(self.event_loop)
#                self.event_loop.run_until_complete(
 #                   self.mqtt_client.connect(self.mqtt_settings["MQTT_BROKER"], self.mqtt_settings["MQTT_PORT"],keepalive=self.mqtt_settings["MQTT_KEEPALIVE"], version=MQTTv311)
  #              )
                logger.warning ('Looping forever')
                self.event_loop.run_forever()
                logger.warning ('Event loop stopped')
                #self.session.close()
            except Exception as e:
                logger.error ('Error in event loop {}'.format(e))

        self.event_loop = asyncio.new_event_loop()

        logger.warning("Starting MQTT thread")
        self._ws_thread = threading.Thread(target=start, args=())

        self._ws_thread.daemon = True
        self._ws_thread.start()

        future = asyncio.run_coroutine_threadsafe(
            self.mqtt_client.connect(self.mqtt_settings["MQTT_BROKER"], self.mqtt_settings["MQTT_PORT"],keepalive=self.mqtt_settings["MQTT_KEEPALIVE"], version=MQTTv311),
            self.event_loop
        )

    def publish(self, topic, payload, retain, qos):
        MQTT_Base.publish(self, topic, payload, retain, qos)

        if self.mqtt_connected is True:
            wrapped = functools.partial(
                self.mqtt_client.publish,topic, payload, retain=retain, qos=qos
            )
            self.event_loop.call_soon_threadsafe(wrapped)
        else:
            logger.warning(
                "Device MQTT publish NOT CONNECTED: {}, retain {}, qos {}, payload: {}".format(
                    topic, retain, qos, payload
                )
            )



#        future = asyncio.run_coroutine_threadsafe(
 #           self.mqtt_client.publish(topic, payload, retain=retain, qos=qos),
  #          self.event_loop
   #     )

    def subscribe(self, topic, qos):  # subclass to provide
        MQTT_Base.subscribe(self, topic, qos)
        self.mqtt_client.subscribe(topic, qos)

    def unsubscribe(self, topic):  # subclass to provide
        MQTT_Base.unsubscribe(self, topic)
        self.mqtt_client.unsubscribe(topic)

    def set_will(self, will, topic, retain, qos):
        MQTT_Base.set_will(self, will, topic, retain, qos)
        #self.mqtt_client.will_set(will, topic, retain, qos)

    def _on_connect(self, client, flags, rc, properties):
        logger.info("MQTT On Connect: {}".format(rc))
        self.mqtt_connected = rc == 0

    def _on_message(self, client, topic, payload, qos, properties):
        #topic = msg.topic
        #payload = msg.payload.decode("utf-8")
        MQTT_Base._on_message(self, topic, payload, False , qos)

    def _on_disconnect(self, client, packet, exc=None):
        self.mqtt_connected = False  # note, change this uses the property setter, do not really need to catch this in the base class
        logger.warning(
            "MQTT Disconnection  {} {} {}".format(
                client, packet, exc
            )
        )
        MQTT_Base._on_disconnect(self, 0)




from .property_switch import Property_Switch


tags = ["Button"]


class Property_Button(Property_Switch):
    def __init__(
        self,
        node,
        id="button",
        name="Button",
        settable=False,
        retained=True,
        qos=1,
        unit=None,
        data_type=None,
        data_format=None,
        value="OFF",
        set_value=None,
        tags=tags,
        meta={},
    ):
        super().__init__(
            node,
            id,
            name,
            settable,
            retained,
            qos,
            unit,
            data_type,
            data_format,
            value,
            set_value,
            tags,
            meta,
        )

    def push(self):
        self.value = "ON"
        self.value = "OFF"



import time
from threading import Event, Thread
import traceback

import logging

logger = logging.getLogger(__name__)


class Repeating_Timer(object):

    """Repeat `function` every `interval` seconds."""

    def __init__(self, interval):
        self.interval = float(interval)
        self.event = Event()
        self.thread = Thread(target=self._target)
        self.thread.daemon = True
        self.thread.start()
        self.callbacks = []

    def _target(self):
        while not self.event.wait(self.interval):
            for callback in self.callbacks:
                try:
                    callback()
                except Exception as e:
                    logger.error("Error in timer callback: {}  {}".format(e,traceback.format_exc()))

    def add_callback(self, callback):
        self.callbacks.append(callback)

    def stop(self):
        self.event.set()
        self.thread.join()




from .node_base import Node_Base

from homie.node.property.property_enum import Property_Enum


class Node_State(Node_Base):
    def __init__(
        self,
        device,
        id="state",
        name="State",
        type_="state",
        retain=True,
        qos=1,
        state_values=None,
        set_state=None,
    ):
        assert state_values
        assert set_state

        super().__init__(device, id, name, type_, retain, qos)

        self.add_property(
            Property_Enum(
                self, "state", "State", data_format=state_values, set_value=set_state
            )
        )

    def update_state(self, state):
        self.get_property("state").value = state




#!/usr/bin/env python

from homie.device_temperature_humidity import Device_Temperature_Humidity
from homie.node.property.property_battery import Property_Battery

import logging

logger = logging.getLogger(__name__)


class Device_Temperature_Humidity_Battery(Device_Temperature_Humidity):
    def register_status_properties(self, node):
        super(Device_Temperature_Humidity_Battery, self).register_status_properties(
            node
        )

        self.battery = Property_Battery(node)
        node.add_property(self.battery)

    def update_battery(self, battery):
        logger.info("Updated Battery {}".format(battery))
        self.battery.value = battery




from .node_base import Node_Base

from homie.node.property.property_contact import Property_Contact


class Node_Contact(Node_Base):
    def __init__(
        self, device, id="contact", name="Contact", type_="contact", retain=True, qos=1
    ):
        super().__init__(device, id, name, type_, retain, qos)

        self.add_property(Property_Contact(self, "contact"))

    def update_contact(self, contact):
        self.get_property("contact").value = contact




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="Homie4",
    version="0.3.8",
    description="Homie 4.0.0 Implementation",
    author="Michael Cumming",
    author_email="mike@4831.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mjcumming/homie4",
    keywords=["HOMIE", "MQTT"],
    packages=setuptools.find_packages(exclude=("test",)),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=["paho-mqtt>=1.3.0", "netifaces>=0.10.6"],
)



#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_base import Node_Base

from homie.node.property.property_setpoint import Property_Setpoint
from homie.node.property.property_temperature import Property_Temperature
from homie.node.property.property_humidity import Property_Humidity
from homie.node.property.property_enum import Property_Enum
from homie.node.property.property_string import Property_String

import logging

logger = logging.getLogger(__name__)
logger.setLevel("INFO")

# example
THERMOSTAT_SETTINGS = {
    "current_temperature": 70,
    "current_humidity": 30,
    "cool_setpoint": 70,
    "max_cool_setpoint": 90,
    "min_cool_setpoint": 60,
    "heat_setpoint": 70,
    "max_heat_setpoint": 90,
    "min_heat_setpoint": 60,
    "fan_mode": "On",
    "fan_modes": ["On", "Auto"],
    "hold_mode": "Schedule",
    "hold_modes": ["Schedule", "Temporary", "Permanent"],
    "system_mode": "Off",
    "system_modes": ["Off", "Heat", "Cool", "Auto"],
    "system_status": "Idle",
    "units": "F",
}


class Device_Thermostat(Device_Base):
    def __init__(
        self,
        device_id=None,
        name=None,
        homie_settings=None,
        mqtt_settings=None,
        thermostat_settings=THERMOSTAT_SETTINGS,
    ):

        super().__init__(device_id, name, homie_settings, mqtt_settings)

        node = Node_Base(self, "controls", "Controls", "controls")
        self.add_node(node)

        if "heat_setpoint" in thermostat_settings:
            heat_setpt_limits = "{}:{}".format(
                thermostat_settings["min_heat_setpoint"],
                thermostat_settings["max_heat_setpoint"],
            )
            self.heat_setpoint = Property_Setpoint(
                node,
                id="heatsetpoint",
                name="Heat Setpoint",
                data_format=heat_setpt_limits,
                unit=thermostat_settings["units"],
                value=thermostat_settings["heat_setpoint"],
                set_value=self.set_heat_setpoint,
            )
            node.add_property(self.heat_setpoint)

        if "cool_setpoint" in thermostat_settings:
            cool_setpt_limits = "{}:{}".format(
                thermostat_settings["min_cool_setpoint"],
                thermostat_settings["max_cool_setpoint"],
            )
            self.cool_setpoint = Property_Setpoint(
                node,
                id="coolsetpoint",
                name="Cool Setpoint",
                data_format=cool_setpt_limits,
                unit=thermostat_settings["units"],
                value=thermostat_settings["cool_setpoint"],
                set_value=self.set_cool_setpoint,
            )
            node.add_property(self.cool_setpoint)

        if "system_mode" in thermostat_settings:
            self.system_mode = Property_Enum(
                node,
                id="systemmode",
                name="System Mode",
                data_format=",".join(thermostat_settings["system_modes"]),
                value=thermostat_settings["system_mode"],
                set_value=self.set_system_mode,
            )
            node.add_property(self.system_mode)

        if "fan_mode" in thermostat_settings:
            self.fan_mode = Property_Enum(
                node,
                id="fanmode",
                name="Fan Mode",
                data_format=",".join(thermostat_settings["fan_modes"]),
                value=thermostat_settings["fan_mode"],
                set_value=self.set_fan_mode,
            )
            node.add_property(self.fan_mode)

        if "hold_mode" in thermostat_settings:
            self.hold_mode = Property_Enum(
                node,
                id="holdmode",
                name="Hold Mode",
                data_format=",".join(thermostat_settings["hold_modes"]),
                value=thermostat_settings["hold_mode"],
                set_value=self.set_hold_mode,
            )
            node.add_property(self.hold_mode)

        node = Node_Base(self, "status", "Status", "status")
        self.add_node(node)

        self.current_temperture = Property_Temperature(
            node,
            unit=thermostat_settings["units"],
            value=thermostat_settings["current_temperature"],
        )
        node.add_property(self.current_temperture)

        self.current_humidity = Property_Humidity(
            node, value=thermostat_settings["current_humidity"]
        )
        node.add_property(self.current_humidity)

        self.system_status = Property_String(
            node,
            id="systemstatus",
            name="System Status",
            value=thermostat_settings["system_status"],
        )
        node.add_property(self.system_status)

        self.start()

    def update_heat_setpoint(self, value):
        self.heat_setpoint.value = value

    def update_cool_setpoint(self, value):
        self.cool_setpoint.value = value

    def update_system_mode(self, value):
        self.system_mode.value = value

    def update_system_status(self, value):
        self.system_status.value = value

    def update_fan_mode(self, value):
        self.fan_mode.value = value

    def update_hold_mode(self, value):
        self.hold_mode.value = value

    def update_current_temperature(self, value):
        self.current_temperture.value = value

    def update_current_humidity(self, value):
        self.current_humidity.value = value

    def set_heat_setpoint(self, value):
        logger.debug("Heat Set {}".format(value))

    def set_cool_setpoint(self, value):
        logger.debug("Cool Set {}".format(value))

    def set_system_mode(self, value):
        logger.debug("Mode Set {}".format(value))

    def set_fan_mode(self, value):
        logger.debug("Fan Set {}".format(value))

    def set_hold_mode(self, value):
        logger.debug("Hold Set {}".format(value))
